#pragma once

#include <memory>
#include <curl/curl.h>

#include "info.h"
#include "slist.h"
#include "option.h"

namespace cppcurl
{

    struct easy
    {
    private:
        using handle_ptr_u = std::unique_ptr<CURL, decltype(&curl_easy_cleanup)>;

    private:
        handle_ptr_u    _handle;
        slist           _header;

    public:
        /**
         * @brief Constructor.
         */
        inline easy();

        /**
         * @brief Get the internal curl easy handle.
         */
        inline CURL* handle() const;

        /**
         * @brief Reset all options of the easy handle.
         */
        inline void reset();

        /**
         * @brief Perform the action configured in this easy handle.
         */
        inline void perform();

        /**
         * @brief Add header to easy handle.
         */
        inline void add_header(const std::string& s);

        /**
         * @brief Add option to the easy handle.
         *
         * @param[in] opt   Option to add to the handle.
         */
        template<CURLoption T_option, typename T_value>
        inline void set_option(const __impl::option<T_option, T_value>& opt);

        /**
         * @brief Add option to the easy handle.
         *
         * @tparam    T_option  Option to add to the handle.
         * @param[in] val       Value of option to add to the handle.
         */
        template<CURLoption T_option>
        inline void set_option(const typename option<T_option>::value_type& val);

        /**
         * @brief Get an info from the easy handle.
         */
        template<CURLINFO T_info>
        inline typename info<T_info>::value_type get_info() const;
    };

}

#include "easy.inl"
