#pragma once

#include <curl/curl.h>
#include <cppcurl/config.h>

#ifdef CPPCURL_HAS_CPPCORE
    #include <cppcore/misc/exception.h>
#else
    #include <exception>
#endif

namespace cppcurl
{

    /**
     * @brief Exception to represent curl errors.
     */
    template<typename T_error>
    struct exception
    #ifdef CPPCURL_HAS_CPPCORE
        : public ::cppcore::exception
    #else
        : public std::exception
    #endif
    {
    public:
        /**
         * @brief Throw an exception if the passed error is not OK.
         */
        static inline void throw_error(T_error err, const std::string& msg);

    public:
        T_error             error;
        const std::string   error_str;

        /**
         * @brief Constructor.
         */
        inline exception(T_error err, const std::string& msg);

    #ifdef CPPCURL_HAS_CPPCORE
    protected:
        /**
         * @brief Print the message of the exception to the passed stream.
         *
         * @param[in] os    Stream to print message of the exception to.
         */
        inline void print_message(std::ostream& os) const override;

    #else
    private:
        const std::string   _what;

    public:
        /**
         * @brief Get the exception message.
         */
        inline const char* what() const throw() override;
    #endif
    };

    using easy_exception = exception<CURLcode>;

    using multi_exception = exception<CURLMcode>;
}
