#pragma once

#include "exception.h"

namespace cppcurl
{

    namespace __impl
    {

        template<typename T_error>
        struct get_error_str
        {
            inline std::string operator()(T_error err) const = delete;
        };

        template<>
        struct get_error_str<CURLcode>
        {
            inline std::string operator()(CURLcode err) const
                { return curl_easy_strerror(err); }
        };

        template<>
        struct get_error_str<CURLMcode>
        {
            inline std::string operator()(CURLMcode err) const
                { return curl_multi_strerror(err); }
        };

    }

    /* exception */

    template<typename T_error>
    void exception<T_error>
        ::throw_error(T_error err, const std::string& msg)
    {
        if (static_cast<bool>(err))
            throw exception(err, msg);
    }

#ifdef CPPCURL_HAS_CPPCORE
    template<typename T_error>
    exception<T_error>
        ::exception(T_error err, const std::string& msg)
        : ::cppcore::exception (msg)
        , error     (err)
        , error_str (__impl::get_error_str<T_error>()(err))
        { }

    template<typename T_error>
    void exception<T_error>
        ::print_message(std::ostream& os) const
    {
        os  << error_str
            << '('
            << cppcurl_convert_cast<int>(error)
            << "): "
            << message;
    }
#else
    template<typename T_error>
    exception<T_error>
        ::exception(T_error err, const std::string& msg)
        : _what     (__impl::get_error_str<T_error>()(err) + "(" + std::to_string(cppcurl_convert_cast<int>(err)) + "): " + msg)
        , error     (err)
        , error_str (__impl::get_error_str<T_error>()(err))
        { }

    template<typename T_error>
    const char* exception<T_error>
        ::what() const throw()
    {
        return _what.c_str();
    }
#endif

}
