#pragma once

#include "multi.h"

#include "easy.inl"
#include "global.inl"
#include "exception.inl"

namespace cppcurl
{

    /* multi */

    multi::multi()
        : _handle(nullptr, &curl_multi_cleanup)
    {
        if (!global::is_initialized())
            throw multi_exception(CURLM_BAD_HANDLE, "libcurl was not initialized!");
        _handle.reset(curl_multi_init());
        if (!_handle)
            throw multi_exception(CURLM_OUT_OF_MEMORY, "Unable to create multi handle");
    }

    CURLM * multi::handle() const
        { return _handle.get(); }

    void multi::add_handle(const easy& e)
    {
        multi_exception::throw_error(
            curl_multi_add_handle(_handle.get(), e.handle()),
            "Unable to add easy handle to multi handle");
    }

    void multi::remove_handle(const easy& e)
    {
        if (!e.handle())
            return;
        multi_exception::throw_error(
            curl_multi_remove_handle(_handle.get(), e.handle()),
            "Unable to remove easy handle from multi handle");
    }

    int multi::prepare_fdsets(fd_set& read, fd_set& write, fd_set& except) const
    {
        int ret = -1;
        multi_exception::throw_error(
            curl_multi_fdset(_handle.get(), &read, &write, &except, &ret),
            "Unable to get the file descriptor sets from multi handle");
        return ret;
    }

    int multi::perform()
    {
        int ret = 0;
        multi_exception::throw_error(
            curl_multi_perform(_handle.get(), &ret),
            "Unable to perform the operations on the multi handle");
        return ret;
    }

    long multi::timeout() const
    {
        long ret;
        multi_exception::throw_error(
            curl_multi_timeout(_handle.get(), &ret),
            "Unable to get timeout from multi handle");
        return ret;
    }

    CURLMsg * multi::read_info() const
    {
        int i;
        return curl_multi_info_read(_handle.get(), &i);
    }

    template<typename T_pred>
    void multi::read_infos(const T_pred&& pred) const
    {
        CURLMsg * msg;
        while ((msg = read_info()))
            pred(*msg);
    }

}
