#pragma once

#include <curl/curl.h>

#include "slist.h"

namespace cppcurl
{

    namespace __impl
    {

        template<CURLoption T_option, typename T_value>
        struct option
        {
        public:
            using value_type = T_value;

            static constexpr CURLoption id = T_option;

        public:
            value_type value;

        public:
            inline option(const T_value& p_value)
                : value(p_value)
                { }
        };

    }

    using data_callback = size_t (*) (char*, size_t, size_t, void*);

    template<CURLoption T_option>
    struct option;

#define cppcurl_define_option(opt, type) \
    template<> \
    struct option<opt> \
        : public __impl::option<opt, type> \
        { using __impl::option<opt, type>::option; }

    cppcurl_define_option(CURLOPT_HTTPHEADER,       const slist&);
    cppcurl_define_option(CURLOPT_PRIVATE,          void*);
    cppcurl_define_option(CURLOPT_POST,             bool);
    cppcurl_define_option(CURLOPT_READFUNCTION,     data_callback);
    cppcurl_define_option(CURLOPT_READDATA,         void*);
    cppcurl_define_option(CURLOPT_WRITEFUNCTION,    data_callback);
    cppcurl_define_option(CURLOPT_WRITEDATA,        void*);
    cppcurl_define_option(CURLOPT_URL,              std::string);
    cppcurl_define_option(CURLOPT_POSTFIELDSIZE,    long);
    cppcurl_define_option(CURLOPT_TIMEOUT,          long);
    cppcurl_define_option(CURLOPT_FOLLOWLOCATION,   bool);

#undef cppcurl_define_option
}
